@@ -151,6 +151,7 @@ fn test_ser_de() {
     test("{x}+sin(2.0*{y})", "2.0*cos(2.0*{y})");
     test("1.0/{x}+cos({y})*2.0", "2.0*-(sin({y}))");
     test("{y}*{x}*2.0", "2.0*{x}");
+    test("3.0", "0.0");    
 }
 
 #[test]
@@ -166,7 +167,7 @@ fn test_ser_de_non_float() {
                         BinOp {
                             apply: |a: i32, b: i32| a % b,
                             prio: 1,
-                            is_commutative: false
+                            is_commutative: false,
                         },
                     ),
                     Operator::make_bin(
@@ -174,7 +175,7 @@ fn test_ser_de_non_float() {
                         BinOp {
                             apply: |a: i32, b: i32| a / b,
                             prio: 1,
-                            is_commutative: false
+                            is_commutative: false,
                         },
                     ),
                 ]
