@@ -1,4 +1,4 @@
-#![doc(html_root_url = "https://docs.rs/exmex/0.11.1")]
+#![doc(html_root_url = "https://docs.rs/exmex/0.11.2")]
 //! Exmex is a fast, simple, and **ex**tendable **m**athematical **ex**pression evaluator
 //! with the ability to compute [partial derivatives](FlatEx::partial) of expressions.  
 //!
@@ -8,40 +8,42 @@
 //! # fn main() -> Result<(), Box<dyn Error>> {
 //! #
 //! use exmex;
-//! assert!((exmex::eval_str::<f64>("1.5 * ((cos(2*PI) + 23.0) / 2.0)")? - 18.0).abs() < 1e-12);
+//! let eval_result = exmex::eval_str::<f64>("1.5 * ((cos(2*π) + 23.0) / 2.0)")?;
+//! assert!((eval_result - 18.0).abs() < 1e-12);
 //! #
 //! #     Ok(())
 //! # }
 //! ```
 //! For floats, we have a list of predifined operators containing
-//! `^`, `*`, `/`, `+`, `-`, `sin`, `cos`, `tan`, `exp`, `log`, and `log2`. The full list is
-//! defined in [`DefaultOpsFactory`](DefaultOpsFactory). Further, the constants π and Euler's number can be
-//! used via `PI` and `E`, respectively. Library users can also create their
+//! `^`, `*`, `/`, `+`, `-`, `sin`, `cos`, `tan`, `exp`, `log`, and `log2`. Further, the constants π 
+//! and Euler's number can be used via `π`/`PI` and `E`, respectively. The full list is
+//! defined in [`DefaultOpsFactory`](DefaultOpsFactory). Library users can also create their
 //! own operators and constants as shown below in the section about extendability.
 //!
 //! ## Variables
 //!
 //! To define variables we can use strings that are not in the list of operators as shown in the following expression.
-//! Additionally, variables should consist only of letters, numbers, and underscores. More precisely, they need to fit the
-//! regular expression
-//! ```r"^[a-zA-Z_]+[a-zA-Z_0-9]*"```.
+//! Additionally, variables should consist only of letters, greek letters, numbers, and underscores. More precisely, they
+//! need to fit the regular expression `r"[a-zA-Zα-ωΑ-Ω_]+[a-zA-Zα-ωΑ-Ω_0-9]*"`, if they are not between curly brackets.
+//!
 //! Variables' values are passed as slices to [`eval`](Express::eval).
 //! ```rust
 //! # use std::error::Error;
 //! # fn main() -> Result<(), Box<dyn Error>> {
 //! #
 //! use exmex::prelude::*;
-//! let to_be_parsed = "log(z) + 2* (-z^2 + sin(4*y))";
+//! let to_be_parsed = "α * log(z) + 2* (-z^2 + sin(4*y))";
 //! let expr = exmex::parse::<f64>(to_be_parsed)?;
-//! assert!((expr.eval(&[3.7, 2.5])? - 14.992794866624788 as f64).abs() < 1e-12);
+//! assert!((expr.eval(&[3.7, 2.5, 1.0])? - 14.992794866624788 as f64).abs() < 1e-12);
 //! #
 //! #     Ok(())
 //! # }
 //! ```
 //! The `n`-th number in the slice corresponds to the `n`-th variable. Thereby, the
-//! alphatical order of the variables is relevant. In this example, we have `y=3.7` and `z=2.5`.
+//! alphabetical order of the variables is relevant. More precisely, the order is defined by the way how Rust sorts strings.
+//! In the example above we have `y=3.7`, `z=2.5`, and `α=1`. Note that `α` is the Greek letter Alpha.
 //! If variables are between curly brackets, they can have arbitrary names, e.g.,
-//! `{456/549*(}`, `{x}`, and confusingly even `{x+y}` are valid variable names as shown in the following.
+//! `{456/549*(}`, `{x}`, and also `{👍+👎}` are valid variable names as shown in the following.
 //! ```rust
 //! # use std::error::Error;
 //! # fn main() -> Result<(), Box<dyn Error>> {
@@ -49,16 +51,80 @@
 //! use exmex::prelude::*;
 //! let x = 2.1f64;
 //! let y = 0.1f64;
-//! let to_be_parsed = "log({x+y})";  // {x+y} is the name of one(!) variable 😕.
+//! let to_be_parsed = "log({👍+👎})";  // {👍+👎} is the name of one variable 😕.
 //! let expr = exmex::parse::<f64>(to_be_parsed)?;
 //! assert!((expr.eval(&[x+y])? - 2.2f64.ln()).abs() < 1e-12);
 //! #
 //! #     Ok(())
 //! # }
 //! ```
-//! The value returned by [`parse`](parse) implements the [`Express`](Express) trait
-//! and is an instance of the struct [`FlatEx`](FlatEx).
+//! The value returned by [`parse`](parse) is an instance of the struct [`FlatEx`](FlatEx) 
+//! that implements the [`Express`](Express) trait. Moreover, [`FlatEx`](FlatEx) and 
+//! [`Express`](Express) are the only items made accessible by the wildcard import from
+//! [`prelude`](prelude).
 //!
+//! ## Partial Derivatives
+//!
+//! For default operators, expressions can be transformed into their partial derivatives
+//! again represented by expressions. To this end, there exists the method [`partial`](Express::partial).
+//! ```rust
+//! # use std::error::Error;
+//! # fn main() -> Result<(), Box<dyn Error>> {
+//! #
+//! use exmex::prelude::*;
+//! let expr = exmex::parse::<f64>("x^2 + y^2")?;
+//! let dexpr_dx = expr.clone().partial(0)?;
+//! let dexpr_dy = expr.partial(1)?;
+//! assert!((dexpr_dx.eval(&[3.0, 2.0])? - 6.0).abs() < 1e-12);
+//! assert!((dexpr_dy.eval(&[3.0, 2.0])? - 4.0).abs() < 1e-12);
+//! #
+//! #     Ok(())
+//! # }
+//! ```
+//!
+//! ## Owned Expression
+//! You cannot return all expression types from a function without a lifetime parameter.
+//! For instance, expressions that are instances of [`FlatEx`](FlatEx) keep `&str`s instead of
+//! `String`s of variable or operator names to make faster parsing possible.
+//! ```rust
+//! # use std::error::Error;
+//! # fn main() -> Result<(), Box<dyn Error>> {
+//! #
+//! use exmex::prelude::*;
+//! use exmex::ExResult;
+//! fn create_expr<'a>() -> ExResult<FlatEx::<'a, f64>> {
+//! //              |                          |
+//! //              lifetime parameter necessary
+//!
+//!     let to_be_parsed = "log(z) + 2* (-z^2 + sin(4*y))";
+//!     exmex::parse::<f64>(to_be_parsed)
+//! }
+//! let expr = create_expr()?;
+//! assert!((expr.eval(&[3.7, 2.5])? - 14.992794866624788 as f64).abs() < 1e-12);
+//! #
+//! #     Ok(())
+//! # }
+//! ```
+//! If you are willing to pay the price of roughly doubled parsing times, you can
+//! obtain an expression that is an instance of [`OwnedFlatEx`](OwnedFlatEx) and owns
+//! its strings. Evaluation times should be comparable. However, a lifetime parameter is
+//! not needed anymore as shown in the following.
+//! ```rust
+//! # use std::error::Error;
+//! # fn main() -> Result<(), Box<dyn Error>> {
+//! #
+//! use exmex::{ExResult, Express, OwnedFlatEx};
+//! fn create_expr() -> ExResult<OwnedFlatEx::<f64>> {
+//!     let to_be_parsed = "log(z) + 2* (-z^2 + sin(4*y))";
+//!     OwnedFlatEx::<f64>::from_str(to_be_parsed)
+//! }
+//! let expr_owned = create_expr()?;
+//! assert!((expr_owned.eval(&[3.7, 2.5])? - 14.992794866624788 as f64).abs() < 1e-12);
+//! #
+//! #     Ok(())
+//! # }
+//! ```
+//! 
 //! ## Extendability
 //!
 //! How to use custom operators as well as custom data types of the operands even with
@@ -67,7 +133,7 @@
 //! ### Custom Operators and Constants
 //!
 //! Operators are instances of the struct
-//! [`Operator`](Operator). Constants are also defined in terms of constant operators. More precisely,
+//! [`Operator`](Operator). Constants are defined in terms of constant operators. More precisely,
 //! operators can be
 //! * binary such as `*`,
 //! * unary such as `sin`,
@@ -193,68 +259,6 @@
 //! # }
 //! ```
 //!
-//! ## Partial Derivatives
-//!
-//! For default operators, expressions can be transformed into their partial derivatives
-//! again represented by expressions. To this end, there exists the method [`partial`](Express::partial).
-//! ```rust
-//! # use std::error::Error;
-//! # fn main() -> Result<(), Box<dyn Error>> {
-//! #
-//! use exmex::prelude::*;
-//! let expr = exmex::parse::<f64>("x^2 + y^2")?;
-//! let dexpr_dx = expr.clone().partial(0)?;
-//! let dexpr_dy = expr.partial(1)?;
-//! assert!((dexpr_dx.eval(&[3.0, 2.0])? - 6.0).abs() < 1e-12);
-//! assert!((dexpr_dy.eval(&[3.0, 2.0])? - 4.0).abs() < 1e-12);
-//! #
-//! #     Ok(())
-//! # }
-//! ```
-//!
-//! ## Owned Expression
-//! You cannot return all expression types from a function without a lifetime parameter.
-//! For instance, expressions that are instances of [`FlatEx`](FlatEx) keep `&str`s instead of
-//! `String`s of variable or operator names to make faster parsing possible.
-//! ```rust
-//! # use std::error::Error;
-//! # fn main() -> Result<(), Box<dyn Error>> {
-//! #
-//! use exmex::prelude::*;
-//! use exmex::ExResult;
-//! fn create_expr<'a>() -> ExResult<FlatEx::<'a, f64>> {
-//! //              |                          |
-//! //              lifetime parameter necessary
-//!
-//!     let to_be_parsed = "log(z) + 2* (-z^2 + sin(4*y))";
-//!     exmex::parse::<f64>(to_be_parsed)
-//! }
-//! let expr = create_expr()?;
-//! assert!((expr.eval(&[3.7, 2.5])? - 14.992794866624788 as f64).abs() < 1e-12);
-//! #
-//! #     Ok(())
-//! # }
-//! ```
-//! If you are willing to pay the price of roughly doubled parsing times, you can
-//! obtain an expression that is an instance of [`OwnedFlatEx`](OwnedFlatEx) and owns
-//! its strings. Evaluation times should be comparable. However, a lifetime parameter is
-//! not needed anymore as shown in the following.
-//! ```rust
-//! # use std::error::Error;
-//! # fn main() -> Result<(), Box<dyn Error>> {
-//! #
-//! use exmex::{ExResult, Express, OwnedFlatEx};
-//! fn create_expr() -> ExResult<OwnedFlatEx::<f64>> {
-//!     let to_be_parsed = "log(z) + 2* (-z^2 + sin(4*y))";
-//!     OwnedFlatEx::<f64>::from_str(to_be_parsed)
-//! }
-//! let expr_owned = create_expr()?;
-//! assert!((expr_owned.eval(&[3.7, 2.5])? - 14.992794866624788 as f64).abs() < 1e-12);
-//! #
-//! #     Ok(())
-//! # }
-//! ```
-//!
 //! ## Priorities and Parentheses
 //! In Exmex-land, unary operators always have higher priority than binary operators, e.g.,
 //! `-2^2=4` instead of `-2^2=-4`. Moreover, we are not too strict regarding parentheses.
@@ -298,10 +302,6 @@
 //! [`Serialize`](https://docs.serde.rs/serde/de/trait.Serialize.html) are implemented for
 //! both, [`FlatEx`](FlatEx) and [`OwnedFlatEx`](OwnedFlatEx).
 //!
-//! ## Unicode
-//! Unicode input strings are currently not supported 😕 but might be added in the
-//! future 😀.
-//!
 
 use std::{fmt::Debug, str::FromStr};
 
@@ -368,13 +368,14 @@ where
 #[cfg(test)]
 mod tests {
 
+    use std::ops::{BitAnd, BitOr};
+    use std::str::FromStr;
     use std::{iter::once, ops::Range};
 
     use rand::Rng;
     use smallvec::{smallvec, SmallVec};
 
     use crate::expression::deep::DeepEx;
-    use crate::prelude::*;
     use crate::{
         eval_str,
         operators::{BinOp, DefaultOpsFactory, MakeOperators, Operator},
@@ -382,6 +383,7 @@ mod tests {
         util::{assert_float_eq_f32, assert_float_eq_f64},
         ExResult, OwnedFlatEx,
     };
+    use crate::{prelude::*, ExError};
 
     #[test]
     fn test_readme() {
@@ -395,21 +397,23 @@ mod tests {
             // d_xy
             let ddexpr_dxy = dexpr_dx.partial(1)?;
             assert_eq!(format!("{}", ddexpr_dxy), "{x}*2.0");
-            assert_float_eq_f64(ddexpr_dxy.eval(&[2.0, f64::MAX])?, 4.0);
+            let result = ddexpr_dxy.eval(&[2.0, f64::MAX])?;
+            assert!((result - 4.0).abs() < 1e-12);
 
             // d_xyx
             let dddexpr_dxyx = ddexpr_dxy.partial(0)?;
             assert_eq!(format!("{}", dddexpr_dxyx), "2.0");
-            assert_float_eq_f64(dddexpr_dxyx.eval(&[f64::MAX, f64::MAX])?, 2.0);
+            let result = dddexpr_dxyx.eval(&[f64::MAX, f64::MAX])?;
+            assert!((result - 2.0).abs() < 1e-12);
 
             Ok(())
         }
         fn readme() -> ExResult<()> {
-            let result = eval_str("sin(73)")?;
-            assert_float_eq_f64(result, 73f64.sin());
-            let expr = parse::<f64>("2*x^3-4/z")?;
-            let value = expr.eval(&[5.3, 0.5])?;
-            assert_float_eq_f64(value, 289.75399999999996);
+            let result = eval_str::<f64>("sin(73)")?;
+            assert!((result - 73f64.sin()).abs() < 1e-12);
+            let expr = parse::<f64>("2*β^3-4/τ")?;
+            let result = expr.eval(&[5.3, 0.5])?;
+            assert!((result - 289.75399999999996).abs() < 1e-12);
             Ok(())
         }
         fn readme_int() -> ExResult<()> {
@@ -460,6 +464,104 @@ mod tests {
         let expr = FlatEx::<f64>::from_str(sut).unwrap();
         assert_float_eq_f64(expr.eval(&[1.5707963267948966]).unwrap(), 1.0);
     }
+
+    #[test]
+    fn test_variables_non_ascii() {
+        let sut = "5*ς";
+        let expr = FlatEx::<f64>::from_str(sut).unwrap();
+        assert_float_eq_f64(expr.eval(&[1.2]).unwrap(), 6.0);
+
+        let sut = "5*{χ} +  4*log2(log(1.5+γ))*({χ}*-(tan(cos(sin(652.2-{γ}))))) + 3*{χ}";
+        let expr = FlatEx::<f64>::from_str(sut).unwrap();
+        println!("{}", expr);
+        assert_float_eq_f64(expr.eval(&[1.2, 1.0]).unwrap(), 8.040556934857268);
+
+        let sut = "sin({myvwmlf4i😎8eo;w/-sin(a)r_25})";
+        let expr = FlatEx::<f64>::from_str(sut).unwrap();
+        assert_float_eq_f64(expr.eval(&[1.5707963267948966]).unwrap(), 1.0);
+
+        let sut = "((sin({myvar_25✔})))";
+        let expr = FlatEx::<f64>::from_str(sut).unwrap();
+        assert_float_eq_f64(expr.eval(&[1.5707963267948966]).unwrap(), 1.0);
+
+        #[derive(Clone, Copy, Debug, PartialEq, Eq)]
+        struct Thumbs {
+            val: bool,
+        }
+        impl BitOr for Thumbs {
+            type Output = Self;
+            fn bitor(self, rhs: Self) -> Self::Output {
+                Self {
+                    val: self.val || rhs.val,
+                }
+            }
+        }
+        impl BitAnd for Thumbs {
+            type Output = Self;
+            fn bitand(self, rhs: Self) -> Self::Output {
+                Self {
+                    val: self.val && rhs.val,
+                }
+            }
+        }
+        impl FromStr for Thumbs {
+            type Err = ExError;
+            fn from_str(s: &str) -> Result<Self, Self::Err> {
+                if s == "👍" {
+                    Ok(Self { val: true })
+                } else if s == "👎" {
+                    Ok(Self { val: false })
+                } else {
+                    Err(Self::Err {
+                        msg: format!("cannot parse {} to `Thumbs`", s),
+                    })
+                }
+            }
+        }
+        ops_factory!(
+            UnicodeOpsFactory,
+            Thumbs,
+            Operator::make_bin(
+                "ορ",
+                BinOp {
+                    apply: |a, b| a | b,
+                    prio: 0,
+                    is_commutative: true,
+                }
+            ),
+            Operator::make_bin(
+                "ανδ",
+                BinOp {
+                    apply: |a, b| a & b,
+                    prio: 0,
+                    is_commutative: true,
+                }
+            ),
+            Operator::make_constant("γ", Thumbs { val: false })
+        );
+
+        let literal_pattern = "👍|👎";
+
+        let sut = "γ ορ 👍ορ👎";
+        let expr = FlatEx::<_, UnicodeOpsFactory>::from_pattern(sut, literal_pattern).unwrap();
+        assert_eq!(expr.eval(&[]).unwrap(), Thumbs { val: true });
+
+        let sut = "(👍 ανδ👎)ορ 👍";
+        let expr = FlatEx::<_, UnicodeOpsFactory>::from_pattern(sut, literal_pattern).unwrap();
+        assert_eq!(expr.eval(&[]).unwrap(), Thumbs { val: true });
+
+        let sut = "(👍ανδ 👎)οργαβ23";
+        let expr = FlatEx::<_, UnicodeOpsFactory>::from_pattern(sut, literal_pattern).unwrap();
+        assert_eq!(
+            expr.eval(&[Thumbs { val: true }]).unwrap(),
+            Thumbs { val: true }
+        );
+        assert_eq!(
+            expr.eval(&[Thumbs { val: false }]).unwrap(),
+            Thumbs { val: false }
+        );
+    }
+
     #[test]
     fn test_variables() {
         let sut = "sin  ({x})+(((cos({y})   ^  (sin({z})))*log(cos({y})))*cos({z}))";
@@ -793,6 +895,8 @@ mod tests {
     #[test]
     fn test_eval() {
         assert_float_eq_f64(eval_str("2*3^2").unwrap(), 18.0);
+        assert_float_eq_f64(eval_str("cos(PI/2)").unwrap(), 0.0);
+        assert_float_eq_f64(eval_str("cos(π/2)").unwrap(), 0.0);
         assert_float_eq_f64(eval_str("-3^2").unwrap(), 9.0);
         assert_float_eq_f64(eval_str("11.3").unwrap(), 11.3);
         assert_float_eq_f64(eval_str("+11.3").unwrap(), 11.3);
